@@ -48,7 +48,7 @@ mod format1 {
         }
     }
 
-    impl<'a> dump_utils::ArrayVisitor<Mapping> for MappingEmitter<'a> {
+    impl dump_utils::ArrayVisitor<Mapping> for MappingEmitter<'_> {
         fn visit(&self, index: u64, b: ArrayBlock<Mapping>) -> anyhow::Result<()> {
             let cbegin = index as u32 * b.header.max_entries;
             let cend = cbegin + b.header.nr_entries;
@@ -112,7 +112,7 @@ mod format2 {
         }
     }
 
-    impl<'a> dump_utils::ArrayVisitor<Mapping> for MappingEmitter<'a> {
+    impl dump_utils::ArrayVisitor<Mapping> for MappingEmitter<'_> {
         fn visit(&self, index: u64, b: ArrayBlock<Mapping>) -> anyhow::Result<()> {
             let cbegin = index as u32 * b.header.max_entries;
             let cend = cbegin + b.header.nr_entries;
@@ -122,12 +122,10 @@ mod format2 {
                 }
 
                 let mut inner = self.inner.lock().unwrap();
-                let dirty = if let Some(bit) = inner.dirty_bits.contains(cblock as usize) {
-                    bit
-                } else {
-                    // default to dirty if the bitset is damaged
-                    true
-                };
+
+                // default to dirty if the bitset is damaged
+                let dirty = inner.dirty_bits.contains(cblock as usize).unwrap_or(true);
+
                 let m = ir::Map {
                     cblock,
                     oblock: map.oblock,
@@ -149,8 +147,8 @@ struct HintEmitter<'a> {
     valid_mappings: FixedBitSet,
 }
 
-impl<'a> HintEmitter<'a> {
-    pub fn new(emitter: &'a mut dyn MetadataVisitor, valid_mappings: FixedBitSet) -> HintEmitter {
+impl HintEmitter<'_> {
+    pub fn new(emitter: &mut dyn MetadataVisitor, valid_mappings: FixedBitSet) -> HintEmitter<'_> {
         HintEmitter {
             emitter: Mutex::new(emitter),
             valid_mappings,
@@ -158,7 +156,7 @@ impl<'a> HintEmitter<'a> {
     }
 }
 
-impl<'a> dump_utils::ArrayVisitor<Hint> for HintEmitter<'a> {
+impl dump_utils::ArrayVisitor<Hint> for HintEmitter<'_> {
     fn visit(&self, index: u64, b: ArrayBlock<Hint>) -> anyhow::Result<()> {
         let cbegin = index as u32 * b.header.max_entries;
         let cend = cbegin + b.header.nr_entries;
@@ -188,7 +186,7 @@ fn dump_v1_mappings(
     cache_blocks: u32,
     repair: bool,
 ) -> anyhow::Result<FixedBitSet> {
-    let ablocks = collect_array_blocks_with_path(engine.clone(), repair, mapping_root)?;
+    let ablocks = collect_array_blocks_with_path(engine.as_ref(), repair, mapping_root)?;
     let emitter = format1::MappingEmitter::new(cache_blocks as usize, out);
     walk_array_blocks(engine, ablocks, &emitter)?;
 
@@ -205,7 +203,8 @@ fn dump_v2_mappings(
 ) -> anyhow::Result<FixedBitSet> {
     // We need to walk the dirty bitset first.
     let dirty_bits = if let Some(root) = dirty_root {
-        let (bits, errs) = read_bitset_checked(engine.clone(), root, cache_blocks as usize, repair);
+        let (bits, errs) =
+            read_bitset_checked(engine.as_ref(), root, cache_blocks as usize, repair);
         if errs.is_some() && !repair {
             return Err(anyhow!("errors in bitset {}", errs.unwrap()));
         }
@@ -216,7 +215,7 @@ fn dump_v2_mappings(
         return Err(anyhow!("format 2 selected, but no dirty bitset present"));
     };
 
-    let ablocks = collect_array_blocks_with_path(engine.clone(), repair, mapping_root)?;
+    let ablocks = collect_array_blocks_with_path(engine.as_ref(), repair, mapping_root)?;
     let emitter = format2::MappingEmitter::new(cache_blocks as usize, dirty_bits, out);
     walk_array_blocks(engine, ablocks, &emitter)?;
 
@@ -230,7 +229,7 @@ fn dump_hint_array(
     valid_mappings: FixedBitSet,
     repair: bool,
 ) -> anyhow::Result<()> {
-    let ablocks = collect_array_blocks_with_path(engine.clone(), repair, hint_root)?;
+    let ablocks = collect_array_blocks_with_path(engine.as_ref(), repair, hint_root)?;
     let emitter = HintEmitter::new(out, valid_mappings);
     walk_array_blocks(engine, ablocks, &emitter)
 }
@@ -247,7 +246,7 @@ impl<'a> OutputVisitor<'a> {
     }
 }
 
-impl<'a> MetadataVisitor for OutputVisitor<'a> {
+impl MetadataVisitor for OutputVisitor<'_> {
     fn superblock_b(&mut self, sb: &ir::Superblock) -> anyhow::Result<ir::Visit> {
         output_context(self.out.superblock_b(sb))
     }
