@@ -1,5 +1,6 @@
 pub mod base;
 pub mod buffer;
+pub mod buffer_pool;
 pub mod gaps;
 pub mod spindle;
 pub mod sync;
@@ -15,6 +16,9 @@ pub mod async_;
 #[cfg(feature = "io_uring")]
 pub use crate::io_engine::async_::AsyncIoEngine;
 
+#[cfg(feature = "io_uring")]
+pub mod ring_pool;
+
 #[cfg(test)]
 pub mod core;
 
