@@ -6,8 +6,11 @@ pub mod btree;
 pub mod btree_builder;
 pub mod btree_error;
 pub mod btree_iterator;
+pub mod btree_layer_walker;
 pub mod btree_leaf_walker;
+pub mod btree_lookup;
 pub mod btree_merge;
+pub mod btree_utils;
 pub mod btree_walker;
 pub mod space_map;
 pub mod unpack;
