@@ -12,6 +12,7 @@ use crate::grid_layout::GridLayout;
 use crate::io_engine::SECTOR_SHIFT;
 use crate::io_engine::*;
 use crate::pdata::btree::{self, *};
+use crate::pdata::btree_utils::*;
 use crate::pdata::btree_walker::*;
 use crate::pdata::space_map::common::SMRoot;
 use crate::pdata::space_map::*;
@@ -38,18 +39,22 @@ pub enum OutputField {
     MappedBlocks,
     ExclusiveBlocks,
     SharedBlocks,
+    HighestMappedBlock,
 
     MappedSectors,
     ExclusiveSectors,
     SharedSectors,
+    HighestMappedSector,
 
     MappedBytes,
     ExclusiveBytes,
     SharedBytes,
+    HighestMappedByte,
 
     Mapped,
     Exclusive,
     Shared,
+    HighestMapped,
 
     TransactionId,
     CreationTime,
@@ -67,18 +72,22 @@ impl FromStr for OutputField {
             "MAPPED_BLOCKS" => Ok(MappedBlocks),
             "EXCLUSIVE_BLOCKS" => Ok(ExclusiveBlocks),
             "SHARED_BLOCKS" => Ok(SharedBlocks),
+            "HIGHEST_BLOCK" => Ok(HighestMappedBlock),
 
             "MAPPED_SECTORS" => Ok(MappedSectors),
             "EXCLUSIVE_SECTORS" => Ok(ExclusiveSectors),
             "SHARED_SECTORS" => Ok(SharedSectors),
+            "HIGHEST_SECTOR" => Ok(HighestMappedSector),
 
             "MAPPED_BYTES" => Ok(MappedBytes),
             "EXCLUSIVE_BYTES" => Ok(ExclusiveBytes),
             "SHARED_BYTES" => Ok(SharedBytes),
+            "HIGHEST_BYTE" => Ok(HighestMappedByte),
 
             "MAPPED" => Ok(Mapped),
             "EXCLUSIVE" => Ok(Exclusive),
             "SHARED" => Ok(Shared),
+            "HIGHEST_MAPPED" => Ok(HighestMapped),
 
             "TRANSACTION" => Ok(TransactionId),
             "CREATE_TIME" => Ok(CreationTime),
@@ -89,32 +98,38 @@ impl FromStr for OutputField {
     }
 }
 
-impl ToString for OutputField {
-    fn to_string(&self) -> String {
+impl std::fmt::Display for OutputField {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         use OutputField::*;
 
-        String::from(match self {
+        let text = match self {
             DeviceId => "DEV",
             MappedBlocks => "MAPPED_BLOCKS",
             ExclusiveBlocks => "EXCLUSIVE_BLOCKS",
             SharedBlocks => "SHARED_BLOCKS",
+            HighestMappedBlock => "HIGHEST_BLOCK",
 
             MappedSectors => "MAPPED_SECTORS",
             ExclusiveSectors => "EXCLUSIVE_SECTORS",
             SharedSectors => "SHARED_SECTORS",
+            HighestMappedSector => "HIGHEST_SECTOR",
 
             MappedBytes => "MAPPED_BYTES",
             ExclusiveBytes => "EXCLUSIVE_BYTES",
             SharedBytes => "SHARED_BYTES",
+            HighestMappedByte => "HIGHEST_BYTE",
 
             Mapped => "MAPPED",
             Exclusive => "EXCLUSIVE",
             Shared => "SHARED",
+            HighestMapped => "HIGHEST_MAPPED",
 
             TransactionId => "TRANSACTION",
             CreationTime => "CREATE_TIME",
             SnapshottedTime => "SNAP_TIME",
-        })
+        };
+
+        write!(f, "{}", text)
     }
 }
 
@@ -126,8 +141,8 @@ pub struct LsTable<'a> {
     data_block_size: u64,
 }
 
-impl<'a> LsTable<'a> {
-    fn new(fields: &'a [OutputField], nr_rows: usize, bs: u32) -> LsTable {
+impl LsTable<'_> {
+    fn new(fields: &[OutputField], nr_rows: usize, bs: u32) -> LsTable<'_> {
         let grid = GridLayout::new_with_size(nr_rows, fields.len());
 
         LsTable {
@@ -154,6 +169,7 @@ impl<'a> LsTable<'a> {
         detail: &DeviceDetail,
         mapped_blocks: u64,
         shared_blocks: u64,
+        highest_mapped_block: u64,
     ) {
         use OutputField::*;
 
@@ -179,10 +195,15 @@ impl<'a> LsTable<'a> {
                 SharedBlocks => shared_blocks,
                 SharedSectors => shared_blocks * bs,
                 SharedBytes | Shared => (shared_blocks * bs) << SECTOR_SHIFT as u64,
+                HighestMappedBlock => highest_mapped_block,
+                HighestMappedSector => (highest_mapped_block + 1) * bs - 1,
+                HighestMappedByte | HighestMapped => {
+                    (((highest_mapped_block + 1) * bs) << SECTOR_SHIFT) - 1
+                }
             };
 
             let cell = match field {
-                Mapped | Exclusive | Shared => {
+                Mapped | Exclusive | Shared | HighestMapped => {
                     let (val, unit) = to_pretty_print_size(val);
                     let mut s = val.to_string();
                     s.push_str(&unit.to_string_short());
@@ -478,39 +499,6 @@ fn read_node(
     read_node_(ctx, metadata_sm, b, depth, ignore_non_fatal, nodes);
 }
 
-/// Gets the depth of a bottom level mapping tree.  0 means the root is a leaf node.
-// FIXME: what if there's an error on the path to the leftmost leaf?
-fn get_depth(ctx: &Context, path: &mut Vec<u64>, root: u64, is_root: bool) -> Result<usize> {
-    use Node::*;
-
-    let b = ctx.engine.read(root).map_err(|_| io_err(path))?;
-    let node =
-        check_and_unpack_node::<BlockTime>(&b, true, is_root).map_err(|e| node_err(path, e))?;
-
-    match node {
-        Internal { values, .. } => {
-            // recurse down to the first good leaf
-            let mut last_err = None;
-            for child in values {
-                if path.contains(&child) {
-                    continue; // skip loops
-                }
-
-                path.push(child);
-                match get_depth(ctx, path, child, false) {
-                    Ok(n) => return Ok(n + 1),
-                    Err(e) => {
-                        last_err = Some(e);
-                    }
-                }
-                path.pop();
-            }
-            Err(last_err.unwrap_or_else(|| node_err(path, NodeError::NumEntriesTooSmall).into()))
-        }
-        Leaf { .. } => Ok(0),
-    }
-}
-
 fn read_internal_nodes(
     ctx: &Context,
     metadata_sm: &Arc<Mutex<dyn SpaceMap + Send + Sync>>,
@@ -523,9 +511,7 @@ fn read_internal_nodes(
         _ => {}
     }
 
-    // FIXME: make get-depth more resilient
-    let mut path = Vec::new();
-    let depth = if let Ok(d) = get_depth(ctx, &mut path, root as u64, true) {
+    let depth = if let Ok(d) = get_depth::<BlockTime>(ctx.engine.as_ref(), root as u64) {
         d
     } else {
         return;
@@ -1073,7 +1059,7 @@ fn count_data_mappings(
     let mut path = Vec::new();
     let roots = btree_to_value_vec(
         &mut path,
-        ctx.engine.clone(),
+        ctx.engine.as_ref(),
         ignore_non_fatal,
         mapping_root,
     )?;
@@ -1172,7 +1158,7 @@ pub fn ls(opts: ThinLsOptions) -> Result<()> {
 
     let mut path = vec![0];
     let details =
-        btree_to_map::<DeviceDetail>(&mut path, ctx.engine.clone(), false, sb.details_root)?;
+        btree_to_map::<DeviceDetail>(&mut path, ctx.engine.as_ref(), false, sb.details_root)?;
 
     let mut table = LsTable::new(&opts.fields, details.len(), sb.data_block_size);
     if !opts.no_headers {
@@ -1183,11 +1169,17 @@ pub fn ls(opts: ThinLsOptions) -> Result<()> {
         let actual_sb = read_superblock(ctx.engine.as_ref(), SUPERBLOCK_LOCATION)?;
         let mapped = count_data_mappings(&ctx, &actual_sb, sb.mapping_root, false)?;
         for ((dev_id, detail), summary) in details.iter().zip(mapped) {
-            table.push_row(*dev_id, detail, summary.nr_mappings, summary.nr_shared);
+            table.push_row(
+                *dev_id,
+                detail,
+                summary.nr_mappings,
+                summary.nr_shared,
+                summary.key_high,
+            );
         }
     } else {
         for (dev_id, detail) in details.iter() {
-            table.push_row(*dev_id, detail, 0, 0);
+            table.push_row(*dev_id, detail, 0, 0, 0);
         }
     }
 
